import express from "express";
import { Request, Response, NextFunction } from "express";
import { Prisma } from "@prisma/client";
import { json } from "body-parser";
import authRoutes from "./routes/auth.routes";
import companyRoutes from "./routes/company.routes";
import companyOperationRoutes from "./routes/company.operation.routes";
import payrollRoutes from "./routes/payRoutes";
import employeeRoutes from "./routes/employee.routes";
import adminDashboardRoutes from "./routes/dashboard/admin.dashboard";
import leaveManagementRoutes from "./routes/leave-management/leave.management.routes";
import statsRoutes from "./routes/stats.routes";
import msgRoutes from "./routes/message.routes";
import rtwRoutes from "./routes/rtw.routes";
import rotaRoutes from "./routes/rota/rota.routes";
import userProfileRoutes from "./routes/user-profile/user-profile.routes";
import { authenticate } from "./middleware/auth.middleware";
import attendanceRoutes from "./routes/attendance/attendance.routes";
import fileUploadRouter from "./controllers/file.manager";
import path from "path";
import recruitmentRouter from "./routes/recruitment/recruitment.routes";
import cors from "cors";

const app = express();

// Middleware
app.use(express.json());

// Routes
app.use("/api", authRoutes);
app.use("/api", companyRoutes);
app.use("/api/company/operation", companyOperationRoutes);
app.use("/api/leave-management", leaveManagementRoutes);
app.use("/api/employee", authenticate, employeeRoutes);
app.use("/api/admin/dashboard", adminDashboardRoutes);
app.use("/api/rota", rotaRoutes);
app.use("/api/profile", userProfileRoutes);
app.use("/api/attendance", attendanceRoutes);
app.use("/api/payroll", payrollRoutes);
app.use("/api/message", msgRoutes);
app.use("/api/rtw", rtwRoutes);
app.use("/api/recruitment", recruitmentRouter);

// Stats route(s); Used for client side static generation,
// dynamic loading, and analytics
app.use("/api/stats", statsRoutes);

// Serve uploaded files statically

const uploadsPath = path.join(__dirname, "../../data/uploads");
app.use("/uploads", cors(), express.static(uploadsPath));

app.use("/api", fileUploadRouter);

app.use((err: any, req: Request, res: Response, next: NextFunction) => {
  // Log the error for debugging

  if (err instanceof Prisma.PrismaClientKnownRequestError) {
    // Handle Prisma errors

    switch (err.code) {
      case "P2002":
        // Unique constraint failed
        return res.status(400).json({
          message: `Duplicate value error:`,
          data: err,
        });
      case "P2003":
        // Foreign key constraint failed
        return res.status(400).json({
          message: `Foreign key constraint error: ${err.message}`,
          data: err,
        });
      case "P2025":
        // Record to update/delete not found
        return res.status(404).json({
          message: `Record not found: ${err.message}`,
          data: err,
        });
      default:
        return res.status(500).json({
          message: `Prisma Error: ${err.message}`,
          data: err,
        });
    }
  } else if (err.name === "JsonWebTokenError") {
    // Handle JWT errors
    res.status(401).json({
      message: "Invalid token",
      data: err,
    });
  } else if (err.name === "TokenExpiredError") {
    // Handle JWT expiration errors
    res.status(401).json({
      message: "Token expired",
      data: err,
    });
  } else {
    // Handle other errors
    res.status(500).json({
      message: "Something went wrong",
      data: err,
    });
  }
});

export default app;
